#!/bin/bash

##########################################################################
#  Burfiks: bayesian filter for the web                                  #
#                                                                        #
#  (c) Copyright 2007, INRIA                                             #
#                                                                        #
#  Authors:                                                              #
#  --------                                                              #
#  Francesco Zappa Nardelli (INRIA, Rocquencourt)                        #
#  Patala Arun Kumar (IIT Kanpur)                                        #
#  Sanchit Garg (IIT Bombay)                                             #
#  Vidyut Ghosal (IIT Kanpur)                                            #
#                                                                        #
#  This file is distributed under the terms of the GNU Public License    #
#  version , http://www.gnu.org/licenses/gpl-2.0.txt                     #
#                                                                        #
##########################################################################

INSTALL_PATH=../sidebar-firefox/burfiks
DOC_PATH=../doc/html

echo "Select the operation to perform: "
echo "1.Only Build an xpi "
echo "2.Build xpi and publish it and sources on the web "
echo "3.Update the home page and manual "
echo "Press either 1,2 or 3: "
read ACTION


if [ "$ACTION" = "1" -o "$ACTION" = "2" -o "$ACTION" = "3" ]; then

	if [ "$ACTION" = "1" ]; then

		cp burfiks_server $INSTALL_PATH/chrome/content/
		cp polipo/polipo $INSTALL_PATH/chrome/content/
		
		# build xpi
		cd $INSTALL_PATH
		zip burfiks.zip chrome/
		zip -g burfiks.zip chrome/content/
		zip -g burfiks.zip chrome/locale/
		zip -g burfiks.zip chrome/locale/en-US/
		zip -g burfiks.zip defaults/
		zip -g burfiks.zip defaults/preferences/
		zip -g burfiks.zip chrome/content/browser-overlay.xul
		zip -g burfiks.zip chrome/content/burfiks.js
		zip -g burfiks.zip chrome/content/cbox-check.gif
		zip -g burfiks.zip chrome/content/cbox.png
		zip -g burfiks.zip chrome/content/confirmPassword.xul
		zip -g burfiks.zip chrome/content/sidebar.xul
		zip -g burfiks.zip chrome/content/treestyle.css
		zip -g burfiks.zip chrome/content/polipo
		zip -g burfiks.zip chrome/content/burfiks_server
		zip -g burfiks.zip chrome/locale/en-US/sidebar.dtd
		zip -g burfiks.zip defaults/preferences/burfiks.js
		zip -g burfiks.zip chrome.manifest
		zip -g burfiks.zip install.rdf
		mv burfiks.zip ../../burfiks.xpi
		cd ../../server
		echo " "
		echo "burfiks.xpi created in parent directory"
		exit
	fi

	echo -n "Enter path to web folder: "
	read HTML_PATH

	if [ -e $HTML_PATH ]; then
		UPDATE_PATH=$HTML_PATH/update.rdf
		burfiksenv=$(svn info | grep "Revision" | sed 's_Revision: __')
		VERSION=1.$burfiksenv
	        if [ "$ACTION" = "3" ]; then
			cp $DOC_PATH/index.html $DOC_PATH/index.html.tmp
			sed "s_<p>The current version is [0-9]*.[0-9]*</p>_<p>The current version is $VERSION</p>_" <$DOC_PATH/index.html.tmp>$DOC_PATH/index.html
			rm $DOC_PATH/index.html.tmp
			sed "s_burfiks-sources.v[0-9]*.[0-9]*.tgz_burfiks-sources.v$VERSION.tgz_" <$DOC_PATH/index.html>$DOC_PATH/index.html.tmp
			mv $DOC_PATH/index.html.tmp $DOC_PATH/index.html

			cp $DOC_PATH/index.html $HTML_PATH
			cp $DOC_PATH/man.html $HTML_PATH
			cp $DOC_PATH/css.css $HTML_PATH

			if [ -e $HTML_PATH/images ]; then
				echo "images folder exists"
			else
				mkdir $HTML_PATH/images
			fi
			
			cp $DOC_PATH/images/burfiks1.png $HTML_PATH/images
			cp $DOC_PATH/images/burfiks2.png $HTML_PATH/images
			cp $DOC_PATH/images/burfiks3.png $HTML_PATH/images
			cp $DOC_PATH/images/burfiks4.png $HTML_PATH/images

			scp $DOC_PATH/index.html burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs
			scp $DOC_PATH/man.html burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs
			scp $DOC_PATH/css.css burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs
			scp $DOC_PATH/images/burfiks1.png burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs/images
			scp $DOC_PATH/images/burfiks2.png burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs/images
			scp $DOC_PATH/images/burfiks3.png burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs/images
			scp $DOC_PATH/images/burfiks4.png burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs/images

			exit
		fi
	else
		echo $HTML_PATH does not exist
		exit
	fi

	cp $INSTALL_PATH/install.rdf $INSTALL_PATH/install.rdf.tmp
	cp $INSTALL_PATH/update.rdf $INSTALL_PATH/update.rdf.tmp
	cp $INSTALL_PATH/chrome/content/burfiks.js $INSTALL_PATH/chrome/content/burfiks.js.tmp
	echo $VERSION
	sed "s|<version>[0-9]*.[0-9]*</version>|<version>$VERSION</version>|" <$INSTALL_PATH/update.rdf.tmp>$INSTALL_PATH/update.rdf
	rm $INSTALL_PATH/update.rdf.tmp
	cp $INSTALL_PATH/update.rdf $HTML_PATH
	scp $INSTALL_PATH/update.rdf burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs

	echo "Select the operating system: "
	echo "1.Linux "
	echo "2.Machintosh "
	echo "3.Windows "
	echo "Press either 1,2 or 3: "
	read CHOICE

	if [ "$CHOICE" = "1" ]; then
		OS_TYPE=linux
		sed "s|<em:id>[a-z_]*@moscova.inria.fr</em:id>|<em:id>linux_burfiks@moscova.inria.fr</em:id>|" <$INSTALL_PATH/install.rdf.tmp>$INSTALL_PATH/install.rdf
		rm $INSTALL_PATH/install.rdf.tmp
		sed "s|<em:version>[0-9]*.[0-9]*</em:version>|<em:version>$VERSION</em:version>|" <$INSTALL_PATH/install.rdf>$INSTALL_PATH/install.rdf.tmp
		mv $INSTALL_PATH/install.rdf.tmp $INSTALL_PATH/install.rdf
		sed "s|[a-z_]*@moscova.inria.fr|linux_burfiks@moscova.inria.fr|" <$INSTALL_PATH/chrome/content/burfiks.js.tmp>$INSTALL_PATH/chrome/content/burfiks.js
		if [ -e $HTML_PATH/linux ]; then
			echo $HTML_PATH/linux already exists
		else
			mkdir $HTML_PATH/linux
		fi
	elif [ "$CHOICE" = "2" ]; then
		OS_TYPE=mac
		sed "s|<em:id>[a-z_]*@moscova.inria.fr</em:id>|<em:id>mac_burfiks@moscova.inria.fr</em:id>|" <$INSTALL_PATH/install.rdf.tmp>$INSTALL_PATH/install.rdf
		rm $INSTALL_PATH/install.rdf.tmp
		sed "s|<em:version>[0-9]*.[0-9]*</em:version>|<em:version>$VERSION</em:version>|" <$INSTALL_PATH/install.rdf>$INSTALL_PATH/install.rdf.tmp
		mv $INSTALL_PATH/install.rdf.tmp $INSTALL_PATH/install.rdf
		sed "s|[a-z_]*@moscova.inria.fr|mac_burfiks@moscova.inria.fr|" <$INSTALL_PATH/chrome/content/burfiks.js.tmp>$INSTALL_PATH/chrome/content/burfiks.js
		if [ -e $HTML_PATH/mac ]; then
			echo $HTML_PATH/mac already exists
		else
			mkdir $HTML_PATH/mac
		fi
	elif [ "$CHOICE" = "3" ]; then
		OS_TYPE=win
		sed "s|<em:id>[a-z_]*@moscova.inria.fr</em:id>|<em:id>win_burfiks@moscova.inria.fr</em:id>|" <$INSTALL_PATH/install.rdf.tmp>$INSTALL_PATH/install.rdf	
		rm $INSTALL_PATH/install.rdf.tmp
		sed "s|<em:version>[0-9]*.[0-9]*</em:version>|<em:version>$VERSION</em:version>|" <$INSTALL_PATH/install.rdf>$INSTALL_PATH/install.rdf.tmp
		mv $INSTALL_PATH/install.rdf.tmp $INSTALL_PATH/install.rdf
		sed "s|[a-z_]*@moscova.inria.fr|win_burfiks@moscova.inria.fr|" <$INSTALL_PATH/chrome/content/burfiks.js.tmp>$INSTALL_PATH/chrome/content/burfiks.js
		if [ -e $HTML_PATH/win ]; then
			echo $HTML_PATH/win already exists
		else
			mkdir $HTML_PATH/win
		fi
	else
		echo "Invalid option entered."
		exit
	fi

	## Distribute xpi

	cd $INSTALL_PATH
	zip burfiks.zip chrome/
	zip -g burfiks.zip chrome/content/
	zip -g burfiks.zip chrome/locale/
	zip -g burfiks.zip chrome/locale/en-US/
	zip -g burfiks.zip defaults/
	zip -g burfiks.zip defaults/preferences/
	zip -g burfiks.zip chrome/content/browser-overlay.xul
	zip -g burfiks.zip chrome/content/burfiks.js
	zip -g burfiks.zip chrome/content/cbox-check.gif
	zip -g burfiks.zip chrome/content/cbox.png
	zip -g burfiks.zip chrome/content/confirmPassword.xul
	zip -g burfiks.zip chrome/content/sidebar.xul
	zip -g burfiks.zip chrome/content/treestyle.css
	zip -g burfiks.zip chrome/content/polipo
	zip -g burfiks.zip chrome/content/burfiks_server
	zip -g burfiks.zip chrome/locale/en-US/sidebar.dtd
	zip -g burfiks.zip defaults/preferences/burfiks.js
	zip -g burfiks.zip chrome.manifest
	zip -g burfiks.zip install.rdf
	mv burfiks.zip $HTML_PATH/$OS_TYPE/burfiks.xpi
	scp $HTML_PATH/$OS_TYPE/burfiks.xpi burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs/$OS_TYPE
	cd ../../server


	sed "s|<em:id>.*[@].*</em:id>|<em:id>burfiks@moscova.inria.fr</em:id>|" <$INSTALL_PATH/install.rdf>$INSTALL_PATH/install.rdf.tmp
	mv $INSTALL_PATH/install.rdf.tmp $INSTALL_PATH/install.rdf
	sed "s|[a-z_]*@moscova.inria.fr|burfiks@moscova.inria.fr|" <$INSTALL_PATH/chrome/content/burfiks.js>$INSTALL_PATH/chrome/content/burfiks.js.tmp
	mv $INSTALL_PATH/chrome/content/burfiks.js.tmp $INSTALL_PATH/chrome/content/burfiks.js

	## Remoce older versions
	rm $HTML_PATH/burfiks-sources.*.tgz
	## Distribute source
	cd ..
	mkdir burfiks-sources.v$VERSION
	mkdir burfiks-sources.v$VERSION/server
	mkdir burfiks-sources.v$VERSION/sidebar-firefox
	mkdir burfiks-sources.v$VERSION/server/bayesian_classifier-1.4
	mkdir burfiks-sources.v$VERSION/server/polipo
	mkdir burfiks-sources.v$VERSION/sidebar-firefox/burfiks
	mkdir burfiks-sources.v$VERSION/sidebar-firefox/burfiks/chrome
	mkdir burfiks-sources.v$VERSION/sidebar-firefox/burfiks/chrome/content
	mkdir burfiks-sources.v$VERSION/sidebar-firefox/burfiks/chrome/locale
	mkdir burfiks-sources.v$VERSION/sidebar-firefox/burfiks/chrome/locale/en-US
	mkdir burfiks-sources.v$VERSION/sidebar-firefox/burfiks/defaults
	mkdir burfiks-sources.v$VERSION/sidebar-firefox/burfiks/defaults/preferences

	cp README burfiks-sources.v$VERSION
	cd server;cp *.ml build.sh Makefile .depend ../burfiks-sources.v$VERSION/server;
	cd bayesian_classifier-1.4;cp *.ml *.mli *.mlp *.mll README ../../burfiks-sources.v$VERSION/server/bayesian_classifier-1.4;
	cd ../polipo;cp *.c *.h *.sample *.html *.man *.texi CHANGES COPYING INSTALL Makefile README README.Windows ../../burfiks-sources.v$VERSION/server/polipo;
	cd ../..
	cd sidebar-firefox/burfiks;cp chrome.manifest install.rdf ../../burfiks-sources.v$VERSION/sidebar-firefox/burfiks;
	cd defaults/preferences;cp burfiks.js ../../../../burfiks-sources.v$VERSION/sidebar-firefox/burfiks/defaults/preferences;
	cd ../../chrome/content;cp burfiks.js browser-overlay.xul cbox-check.gif cbox.png confirmPassword.xul sidebar.xul treestyle.css ../../../../burfiks-sources.v$VERSION/sidebar-firefox/burfiks/chrome/content;
	cd ../locale/en-US;cp sidebar.dtd ../../../../../burfiks-sources.v$VERSION/sidebar-firefox/burfiks/chrome/locale/en-US;
	cd ../../../../../
	tar -cvzf burfiks-sources.tgz burfiks-sources.v$VERSION/
	rm -R burfiks-sources.v$VERSION

	mv burfiks-sources.tgz $HTML_PATH/burfiks-sources.v$VERSION.tgz
	scp $HTML_PATH/burfiks-sources.v$VERSION.tgz burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs
	cd server/

	cp burfiks_server $HTML_PATH/$OS_TYPE
	cp polipo/polipo $HTML_PATH/$OS_TYPE
	scp burfiks_server burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs/$OS_TYPE
	scp polipo/polipo burfiks.gforge.inria.fr:/home/groups/burfiks/htdocs/$OS_TYPE

else
	echo "Invalid option entered."
	exit
fi	
