(**************************************************************************)
(*  Burfiks: bayesian filter for the web                                  *)
(*                                                                        *)
(*  (c) Copyright 2007, INRIA                                             *)
(*                                                                        *)
(*  Authors:                                                              *)
(*  --------                                                              *)
(*  Francesco Zappa Nardelli (INRIA, Rocquencourt)                        *)
(*  Patala Arun Kumar (IIT Kanpur)                                        *)
(*  Sanchit Garg (IIT Bombay)                                             *)
(*  Vidyut Ghosal (IIT Kanpur)                                            *)
(*                                                                        *)
(*  This file is distributed under the terms of the GNU Public License    *)
(*  version 2, http://www.gnu.org/licenses/gpl-2.0.txt                    *)
(*                                                                        *)
(**************************************************************************)

open Auxil
open Types
open Unix
open Str
open State
open Proxy
open Rank


(* network connection management *)

let sock_send = fun fd data ->
  debug "send" data;
  let send_all s =
    let rec send_all_int off len s =
      let r = (send fd s off (len-off) []) in
      if r = (len - off) 
      then ()
      else send_all_int (off + r) len s
    in send_all_int 0 (String.length s) s
  in send_all data


(* *** handle requests from the client *)

let stripcrnl s = Str.global_replace (regexp "[\r\n]") "" s;;

let getline ic =    
  try
  let s = input_line ic in   
  debug "getline" (s^"\n");
  s
  with e -> debug "Error in getline" ((Printexc.to_string e)^"\n");
            raise e
(** kill server**)
let kill_server () =
      (** close server socket??**)
      (match !polipo_pid with 
       | Some pid -> Unix.kill pid Sys.sigint; 
                    debug "kill_session" ("KILLED POLIPO WHICH WAS RUNNING WITH PID "^(string_of_int(pid))^"\n");
       | None -> (debug "kill_session" ("POLIPO NOT KILLED"^"\n")));
      debug "kill_session" ("BURFIKS SERVER KILLED"^"\n");
      Unix.kill (Unix.getpid()) Sys.sigint

(* sockets list management **
** for sending message to all sockets of a user (multiple windows)**)

let socket_list: (Unix.file_descr * string) list ref = ref [];;

let add_socket socket login=
  socket_list := (socket,login)::!socket_list;;

let remove_socket socket=
  socket_list := List.filter (fun (s,l) -> if s=socket then false else true) !socket_list;;

let send_user login msg =
(List.iter (fun (sock,login')->  (if (login'=login) then
                                                    sock_send sock msg;)) !socket_list;);;

let send_all login msg =
(List.iter (fun (sock,_)-> sock_send sock msg;) !socket_list;);;



(** handles the client session **)

let handle_session (ic, oc,s,polipo_socket,polipo_ic,caller ) =
  let kill_session () = 
  debug "kill_session" "user logging off\n";
  users := !users-1;
  try
    remove_socket s;
    Unix.close s ;
    Unix.close !polipo_socket;
    if (!users = 0 & !normal_mode) then (kill_server ())
    else Thread.exit();
  with _ -> if (!users = 0 & !normal_mode) then (kill_server ())
            else Thread.exit(); in

try (
  let (login, profile) = (ref None, ref None) in 
(*  let rec create_psl n =
  match n with
  | 0 -> []
  | _ -> let ps = (ref (Proxy.create_socket ())) in
         let ps_ic = (ref (in_channel_of_descr !ps)) in
         (ps,ps_ic)::(create_psl (n-1)) in
  let pslist = (create_psl 5) in*)
(*  let ps1 = (ref (Proxy.create_socket ())) in
  let ic1 = (ref (in_channel_of_descr !ps1)) in
  let ps2 = (ref (Proxy.create_socket ())) in
  let ic2 = (ref (in_channel_of_descr !ps2)) in*)

  let ps1 = (ref (create_socket ())) in
  let ic1 = (ref (in_channel_of_descr !ps1)) in
  let ps2 = (ref (create_socket ())) in
  let ic2 = (ref (in_channel_of_descr !ps2)) in
  let _ = 
    ( match caller with
    | ADDR_INET (addr,p) -> addr
    | _ -> warning "client connected over ADDR_UNIX"; 
           kill_session (); inet_addr_any) in


  (* *** login check *)
  let input_buffer = getline ic in
  ( match input_buffer with
  | "LOGIN" ->   
      let log = getline ic in
      login := Some log;
      let passwd = getline ic in
      let result = Passwd.check_login log passwd in
      debug "login" ("user "^ log ^" logged in: " ^(string_of_bool result)^"\n");
      if result then (
        sock_send s "LOGIN_SUCCEED \n";
        sock_send s (Passwd.collect_profiles log) )
      else (
        sock_send s "LOGIN_FAILED \n";
	kill_session ();
        )

  | "ADDUSER" -> 
      let log = getline ic in
      login := Some log;
      let passwd = getline ic in
      let result = Passwd.create_user log passwd in
      debug "login" ("user "^ log ^" created: "^(string_of_bool result)^"\n");
      if result then
        (sock_send s "USER_ADDED \n";
        (*sock_send s (Passwd.collect_profiles log);*) )
      else (
        sock_send s "ADD_USER_FAILED \n";
        kill_session ();
      )

  | _ -> warning "client did not login"; kill_session (););

  (* *** send ranked links back to the sidebar *)

  
    let send_back p result (t,u,l) = 
            begin
		let data = "LINKS " ^ (string_of_int result.prob) ^ " " ^ p ^ " " ^ t ^ " " ^ u ^ " " ^ l ^ " " ^ "\n" in
	        sock_send s data;
		let word_data = ref ("KEYS "^result.word_list^" \n")  in
		sock_send s !word_data;
	    end in

  let process_session session =
    try
    ( match !login, !profile with
    | Some login, Some profile ->
    Proxy.send_all_links s ((ps1,ic1),(ps2,ic2))profile session.links (send_back session.url_parent) Rank.rank; 
   (*(
   if not(!no_rank) then 
   (let tmlinks = ref [] in
   List.iter (fun (t,u,l)-> 
		(
                if !slow_server then Unix.sleep 1;
                try
                (let page = Proxy.get_urlbody u ((polipo_socket,polipo_ic),s,1.1) in
                let result =  Rank.rank profile (u,l) page in 
                send_back session.url_parent result (t,u,l))
                with  Interrupt "timeout"-> (debug "session" ("recieved timeout interrupt  \n");
                                        tmlinks:=(t,u,l)::!tmlinks;())
                )) session.links ;
   List.iter (fun (t,u,l)-> 
		(
                if !slow_server then Unix.sleep 1;
                (let page = Proxy.get_urlbody u ((polipo_socket,polipo_ic),s,-1.0) in
                let result =  Rank.rank profile (u,l) page in 
                send_back session.url_parent  result (t,u,l))
                )) !tmlinks ;
    )
    else
    (List.iter (fun (t,u,l)-> 
                if !slow_server then Unix.sleep 1;
                (let page = None in
                let result =  Rank.rank profile (u,l) page in 
                send_back session.url_parent result (t,u,l))
                ) session.links ;))*)
    | _,_ -> ()) 
    with Interrupt x-> Proxy.renew_socket(ps1,ic1);
                       Proxy.renew_socket(ps2,ic2);
                       debug "session" ("recieved interrupt  "^x^"\n");() in


    (match !login with
    | Some log -> add_socket s log;
    | None -> (););
  while true do

    (* *** session formats 
       
       "LINKS \n url_of_parent \n link \n ... link \n END\n" 
       "LOGOUT\n" *)

    let protocol = getline ic in
    
    ( match protocol with

    | "LINKS" ->
        (* - get url of the parent *)
        let parent_url = getline ic in 
        debug "recv" ("received parent_url: "^parent_url^"\n");
        
        (*( match !thread_replying with
        | Some tid -> (
            debug "session" "received new connection\n";
            Event.sync (Event.send interrupt ()) )
        | None -> () );*)
        
        let rec collect_links links =  
          let input_buffer = getline ic in
          debug "recv" ("received link: "^input_buffer^"\n");
          ( match input_buffer with
          | "END" -> debug "session" "received all links\n"; links
          | s ->     
              (* *** format: "target href label" *)
              let tokens = split (regexp " ") input_buffer in
              ( match tokens with
              | target :: href :: label -> 
                  collect_links ((target, href, String.concat " " label)::links)
              | _ -> 
                  warning ("received malformed link: "^s); 
                  collect_links links ) ) in
    
        let session = { url_parent = parent_url; links = collect_links [] } in
        (*thread_replying mutex??*)
(*          thread_replying := Some (Thread.create process_session session); *)
        (*thread_replying := 
	  Some (Thread.create 
		  ( fun () -> 
                    process_session session; 
		    thread_replying := None;
		    Event.poll (Event.receive interrupt) ) ())*)
        process_session session;
         
    | "CREATEPROFILE" ->
        let profile = getline ic in
        ( match !login with
        | Some login ->
            debug "recv" ("create profile: " ^ login ^ " - " ^ profile ^ "\n");
            let result = Passwd.create_profile login profile in
	    if result then
	    begin
              let _ = Rank.create_profile (false,login,profile) in
              send_user login ("PROFILE_ADDED " ^ profile ^ "\n");
	    end
            else
              sock_send s ("PROFILE_ADD_FAILED  \n")
        | _ -> 
            warning "wierd in createprofile"; 
            sock_send s ("PROFILE_ADD_FAILED  \n") )

    | "DELETEPROFILE" ->
        let public= ((getline ic)="PUBLIC") in 
        let prof = (getline ic) in
        ( match !login, !profile with
        | Some login, _ ->
            debug "recv" ("delete profile: " ^ login ^ " - " ^ prof ^ "\n");
            Rank.close_profile (public,login,prof);
            let result = (match public with
             | false -> (Passwd.delete_profile login prof)
             | true -> (Passwd.delete_publicprofile login prof)) in
            if (result) then 
            (profile := None;
               if public then (send_all login ("PROFILE_DELETED " ^login^" "^prof ^ " \n"))
               else (send_user login ("PROFILE_DELETED " ^login^" "^prof ^ " \n")))
            else
             (sock_send s ("PROFILE_DEL_FAILED\n");)
        | _ -> 
            warning "weird in deleteprofile";
            sock_send s ("PROFILE_DEL_FAILED\n") )

    | "SHARE" ->
        let prof = (getline ic) in 
        ( match !login, !profile with
        | Some login, _ -> 
                 Rank.close_all_profiles (false,login,prof);
                 let return = Passwd.move_profile login prof in
                 if (return) then send_all s ("SHARE_SUCCEED "^login^" "^prof^" "^"\n")
                 else sock_send s ("SHARE_FAILED\n")
        | _,_ -> sock_send s ("SHARE_FAILED\n"))

    | "UNSHARE" ->
        let prof = (getline ic) in 
        ( match !login, !profile with
        | Some login, _ -> 
                 Rank.close_all_profiles (true,login,prof);
                 let return = Passwd.unmove_profile login prof in
                 if (return) then send_all s ("UNSHARE_SUCCEED "^login^" "^prof^" "^"\n")
                 else sock_send s ("UNSHARE_FAILED\n")
        | _,_ -> sock_send s ("UNSHARE_FAILED\n"))


    | "DELETEUSER" ->
        ( match !login, !profile with
        | Some login, Some profile ->
            Rank.close_profile profile;
            debug "recv" ("delete user: " ^ login ^ "\n");
            let result = Passwd.delete_user login in
            if result then
            (send_user login ("USER_DELETED\n");)
            else
              sock_send s ("USER_DEL_FAILED\n")
        | _ ->
            warning "weird in deleteuser";
            sock_send s ("USER_DEL_FAILED \n") )

    | "PROFILE_SELECTED" ->
        ( match !login, !profile with
        | Some login, Some profile -> 
            Rank.close_profile profile
        | _ -> () );
        let public = ((getline ic)="PUBLIC") in
        let user = getline ic in
        let prof = (getline ic) in
        ( match !login with
        | Some login ->
            profile := Some (public,user,prof);
            debug "recv" ("profile select: " ^ prof ^ "\n");
            let ret = Rank.open_profile (public,user,prof) in
            if ret then  sock_send s ("PROFILE_SELECT_DONE " ^ prof ^ "\n")
            else sock_send s ("PROFILE_SELECT_FAILED " ^ prof ^ "\n")
        | _ -> 
            warning "weird in profile_selected";
            sock_send s ("PROFILE_SELECT_FAILED " ^ prof ^ "\n") )

    | "CLASSIFIER_CHANGED" ->
        let classifier = getline ic in
        sock_send s ("CLASSIFIER_CHANGE_DONE " ^ classifier ^ "\n")

    | "LOGOUT" ->
        ( match !login, !profile with
        | Some log, Some prof ->
            login := None;
            profile := None;
            debug "login" "LOGGED OUT\n";
            Rank.close_profile prof;
            sock_send s "LOGOUT_SUCCEED\n";
            kill_session ();

        | _ -> 
            warning "weird in logout";
            sock_send s "LOGOUT_SUCCEED \n" )

    | "LOGOUT_ALL" ->
        ( match !login, !profile with
        | Some log, Some prof ->
            login := None;
            profile := None;
            debug "login" "LOGGED OUT\n";
            Rank.close_profile prof;
            send_user log "LOGOUT_SUCCEED\n";
            kill_session ();

        | _ -> 
            warning "weird in logout";
            sock_send s "LOGOUT_SUCCEED \n" )

    | "DUMP" ->
        ( match !login, !profile with
        | Some log, Some prof ->
            let ret = dump_profile prof in
              sock_send s ret;
        | _ -> 
            warning "weird in dump";
            sock_send s "DUMP_FAILED \n" )

    | "CLEAR_CACHE" ->
        let status = Proxy.clear_cache () in
        if status then (sock_send s "CACHE_SUCCEED\n")
        else (sock_send s "CACHE_FAILED\n")

    | "YES" ->
        ( match !login, !profile with
        | Some login, Some profile ->
	  let rec collect_links links =  
          let input_buffer = getline ic in
          (debug "recv" ("yes/no received link: "^input_buffer^"\n");
          ( match input_buffer with
          | "END" -> debug "yes/no session" "received all links\n"; links
          | s -> collect_links (input_buffer::links))) in
          let links = collect_links [] in
          let plinks = ref 0 in
           (try
           List.iter (*(fun u -> ())*)
           (fun url ->
            debug "evaluate" ("YES " ^ url ^ "\n");
            try
            let page = Proxy.get_urlbody url ((polipo_socket,polipo_ic),s,1.1) in
            (match page with
            | Some p -> let evaluation =(Rank.evaluate_url profile url false (Some p)) in
                        if evaluation
                        then plinks:=!plinks+1;
            | None ->   ();)
            with Interrupt "timeout" ->()) links;
            sock_send s ("CHOICE_PROCESSED "^(string_of_int !plinks)^"\n");
          with Interrupt _ -> 
              debug "data on" "interrupt\n";
              sock_send s ("CHOICE_PROCESSED "^(string_of_int !plinks)^"\n"););
        | _ -> () )

    | "NO" ->
        ( match !login, !profile with
        | Some login, Some profile ->
	  let rec collect_links links =  
          let input_buffer = getline ic in
          (debug "recv" ("yes/no received link: "^input_buffer^"\n");
          ( match input_buffer with
          | "END" -> debug "yes/no session" "received all links\n"; links
          | s -> collect_links (input_buffer::links))) in
          let links = collect_links [] in
          let plinks = ref 0 in
           (try
           List.iter (*(fun u -> ())*)
           (fun url ->
            debug "evaluate" ("NO " ^ url ^ "\n");
            try
            let page = Proxy.get_urlbody url ((polipo_socket,polipo_ic),s,1.1) in
            (match page with
            | Some p -> let evaluation =(Rank.evaluate_url profile url true (Some p)) in
                        if evaluation
                        then plinks:=!plinks+1;
            | None ->   ();)
            with Interrupt "timeout" ->()) links;
            sock_send s ("CHOICE_PROCESSED "^(string_of_int !plinks)^"\n");
          with Interrupt _ -> 
              debug "data on" "interrupt\n";
              sock_send s ("CHOICE_PROCESSED "^(string_of_int !plinks)^"\n"););
        | _ -> () )

    | _ -> warning ("client sent malformed request: " ^ protocol) )
  done
)
with e -> 
        debug "threads::" "exception"; 
	sock_send s "LOGOUT_SUCCEED \n";
        print_endline (Printexc.to_string e);
        kill_session ();;

(* *** all together *)

let main () =
  Arg.parse 
    ( Arg.align
       [
        ( "-polipo_bin",
          Arg.String (fun s -> polipo_bin := s),
          " Specify the path of the polipo binary" );
        ( "-normal_mode",
          (*Arg.String (fun s -> match s with
                               | "true" -> normal_mode:=true
                               | "false" -> normal_mode:=false
                               | _ -> ()),*)
	Arg.Set normal_mode,
          " In normal mode server is shutdown if there are no more users logged in" );
        ( "-port",
          Arg.Int (fun p -> server_port := p),
          " Set the port at which the server listens for connections" ); 
        ( "-dir",
          Arg.String (fun s -> burfiks_dir := s),
          " Set where profiles and caches are stored" );
(*        ( "-polipo_running_at",
          Arg.String (fun s -> 
                     polipo_running := Some s;),
          " Connect to an existing polipo instance at port" );*)
        (*( "-proxy_running_at",
          Arg.String (fun s -> 
                     try
                     match (Str.split (Str.regexp ":") s) with
                     | [server;port] ->  proxy_running := Some (server,(int_of_string port));
                     | _ -> proxy_running := None;
                     with _ -> proxy_running := None;),
          " Connect to an existing proxy instance (server:port) " );*)
        ( "-parent_proxy",
          Arg.String (fun s -> parent_proxy := Some s),
          " Connect through parent proxy (server:port) " );
        ( "-proxy_auth",
          Arg.String (fun s -> proxy_auth := Some s),
          " Parent proxy authentication credentials (username:password) " );
        ( "-debug", 
          Arg.String (fun s -> active_debug := s::!active_debug),
          " [debug] Enable debug messages labeled as 's'" );
        ( "-debug_all", 
          Arg.Set active_debug_all,
          " [debug] Enable all debug messages" );
        ( "-slow",
          Arg.Set slow_server,
          " [debug] Emulates a slow server for testing purposes" );
        ( "-norank",
          Arg.Set no_rank,
          " [debug] Turns off ranking for testing purposes" );
      ] )
    ignore
    "Usage: BurfiksServer [-slow] [port n] [-debug s] [-debug_all]";


(*  Proxy.clear_cache ();
  exit 0;*)
  verify_dir (!State.burfiks_dir);
  verify_dir (Filename.concat !State.burfiks_dir "public");

  (* - wait for sessions - *)
  try
  let connection_socket = socket PF_INET SOCK_STREAM 0 in
  setsockopt connection_socket SO_REUSEADDR true;  
  bind connection_socket (ADDR_INET (inet_addr_any, !server_port));
  listen connection_socket 5;
  (*if not !no_rank then begin
    match !proxy_running with
    | None -> Proxy.init ()
    | _ -> ()
  end;*)
  if ((not !no_rank) &(not (Proxy.ping()))) then Proxy.init ();

  while true do
    debug "session" "waiting for a connection\n";
    let (s,caller) = accept connection_socket in
    let out_ch = out_channel_of_descr s in
    let in_ch = in_channel_of_descr s in
    let polipo_socket = ref (Proxy.create_socket ()) in
    let polipo_ic = ref (in_channel_of_descr !polipo_socket) in
    users := !users+1;
    ignore (Thread.create handle_session (in_ch, out_ch, s,polipo_socket,polipo_ic,caller));
  done
  with Unix.Unix_error(_,"bind",_)  ->debug "Server::" "already running or port busy\n";exit 0;;

let _ = 
  print_endline ("Burfiks " ^ Version.version);
  main()
(*   Proxy.init (); *)
(*   Proxy.test_main ();; *)
(* print_endline (string_of_bool (Proxy.ping ()));; *)
 (*
 let _ = Rank.open_profile (true,"a","work") in
 print_endline (dump_profile (true,"a","work"));
 let _ = Rank.close_profile (true,"a","work") in
 exit 0;;*)



