(**************************************************************************)
(*  Burfiks: bayesian filter for the web                                  *)
(*                                                                        *)
(*  (c) Copyright 2007, INRIA                                             *)
(*                                                                        *)
(*  Authors:                                                              *)
(*  --------                                                              *)
(*  Francesco Zappa Nardelli (INRIA, Rocquencourt)                        *)
(*  Patala Arun Kumar (IIT Kanpur)                                        *)
(*  Sanchit Garg (IIT Bombay)                                             *)
(*  Vidyut Ghosal (IIT Kanpur)                                            *)
(*                                                                        *)
(*  This file is distributed under the terms of the GNU Public License    *)
(*  version 2, http://www.gnu.org/licenses/gpl-2.0.txt                    *)
(*                                                                        *)
(**************************************************************************)

open Auxil
open Unix
open Str
open Types 
open State 
(*
1.check for infinite redirection
2.other status codes....201,202,100,101,........
3.sort_links response slow??
4.check version of the http protocol 
same connection ... chage in get_data...content-length
*)

let polipo_status_mtx = Mutex.create () ;;


let create_socket () =
    (*(let (server,port) = (match !proxy_running with
	|None -> ((Unix.inet_addr_of_string "127.0.0.1"),8123)
	|Some (s,p) -> (try
                      ((Unix.gethostbyname s).Unix.h_addr_list.(0),p)
                      with  _ -> ((Unix.inet_addr_of_string "127.0.0.1"),8123)))
                      in*)
    let (server,port) = ((Unix.inet_addr_of_string "127.0.0.1"),8123) in
    let address = Unix.ADDR_INET (server,port) in
    let socket =Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
      Unix.connect socket address;
      socket

let sendstr socket str =
    (Unix.send socket str 0 (String.length str) [])

let renew_socket (polipo_socket,polipo_ic) =
  debug "renew_socket" "\n";
  try
  close_in !polipo_ic;
  Unix.close !polipo_socket;
  polipo_socket := (create_socket ());
  polipo_ic :=  (in_channel_of_descr !polipo_socket)
  with Unix.Unix_error (_,_,_) -> polipo_socket := (create_socket ());
  	polipo_ic :=  (in_channel_of_descr !polipo_socket);;
  
  
(**********************************************************************************)
(** recieve functions **)
let get_close data=
    try
    let _ = Str.search_forward (Str.regexp_case_fold "\\(^Connection:[ ]*\\)\\(close*\\)") data 0 in
    true
    with Not_found -> false
  

let get_status data =
    let _ = Str.search_forward (Str.regexp_case_fold "\\(HTTP/1.[0-1][ ]*\\)\\([0-9]*\\)") data 0 in
    Str.matched_group 2 data;;

let rec recieve socket= 
    (let buffer = String.create 255 in
    let getstr = (Unix.recv socket buffer 0 (String.length buffer) []) in
      if getstr = 0
      then ""
      else (String.sub buffer 0 getstr) ^ (recieve socket))


let rec recieve_length ic len = 
  let buffer = String.create (len) in
  let ret = (input ic buffer 0 (len)) in
(*  print_endline("$$$$$$$#####");
  print_int(len);
  print_endline("");
  print_int(ret);
  print_endline("");
  print_endline(buffer);
  print_endline(string_of_bool ((ret-len)=0));
  print_endline("$$$$$$$$$$$");*)
  if ((ret-len)=0) then  ((*print_endline("$%%$%%$%%%$$$$");*) buffer)
  else ((String.sub buffer 0 ret)^(recieve_length ic (len-ret)));;

  


let rec recieve_headers ic= 
  match (input_line ic) with
  | "\r" -> "\r\n"
  | s ->  (s^"\n"^(recieve_headers ic));;

let rec recieve_chunked ic =
  let len = (input_line ic) in
(*   print_endline(len); *)
  let len2 = (String.sub len 0 ((String.length len)-1)) in
(*   debug "recieve_chunked:len2::" (len2^"\n"); *)
  let len3 = (int_of_string ("0x"^(len2))) in
(*   print_endline ("WWWW"^(string_of_int len3)^"EEE"); *)
  let buffer = String.create (len3) in
  let ret = (recieve_length ic len3) in
  let _ = input_line ic in
(*   print_int(ret); *)
(*   print_endline(""); *)
(*   print_endline(buffer); *)
  match len3 with
  | 0 -> ""
  | _ -> (buffer^(recieve_chunked ic));;

(*let recieve_content_length headers ic =
  let content_length () =
  let reg = Str.regexp_case_fold "\\(Content-Length:[ ]*\\)\\([0-9]*\\)" in
    print_endline("$$"^ret^"$$");
    print_int(String.length ret);
    print_endline("");
    ((int_of_string (ret))) in
  let len =  content_length () in
  let buffer= (recieve_length ic len) in
(*   let _ = input_line ic in *)
  buffer;;*)


let recieve_new str ic =
let return =
(*  debug "recieve_new::str " (str^"\n"); *)
 (let headers = recieve_headers ic in
(*  debug "headers\n" (headers^"\n"); *)
 let is_head_request () = 
    let reg = Str.regexp_case_fold "\\(^HEAD \\)" in
    try Str.search_forward reg str 0; true
    with Not_found -> false in
 let is_chunked_transfer () = 
    let reg = Str.regexp_case_fold "\\(Transfer-Encoding:[ ]*\\)\\(chunked\\)" in
    try Str.search_forward reg headers 0;true
    with Not_found -> false in
 let is_content_length () =
    let reg = Str.regexp_case_fold "\\(Content-Length:[ ]*\\)\\([0-9]*\\)" in
    try Str.search_forward reg headers 0;
    let ret = (Str.matched_group 2 headers) in
    Some (int_of_string (ret))
    with Not_found -> None in
 let is_head = is_head_request () in
(*   print_endline("@@@@@@@@@@@@"^(string_of_bool is_head)^"$$$\n"^headers^"\n"); *)
  match is_head with
  | true -> headers
  | false -> 
	let ret = (is_chunked_transfer ()) in
(*         debug "is_chunked_transfer" (string_of_bool ret); *)
	match ret with
	| true -> headers^(recieve_chunked ic)
	| false -> 
		let ret = (is_content_length ()) in
		match ret with
		| Some len -> (*debug "is_content_length" (string_of_int len);*)headers^(recieve_length ic len)
		| None -> headers) in
  debug "recieve_new::str:: " (str^"\n");
(*   debug "recieve_new::return:: " (return^"\n\n"); *)
  return;;

(**********************************************************************************)




let get_data str ((socket,ic),client_socket,tm) =
(*       debug "get_data:: " str; *)
(*     let socket = create_socket () in *)
(*  debug "test121\n" ""; *)
      sendstr !socket str;
(*  debug "test122\n" ""; *)
    let ret,_,_ = Unix.select (!socket::(client_socket::[])) [] [] (tm) in
     if ((List.length ret)=0) then begin  (*Unix.close !socket;*)(renew_socket(socket,ic);raise (Interrupt "timeout");) end
    else
    begin
    List.iter 
    (fun s -> if (s=client_socket) then 
              begin
              (*Unix.close socket;*)renew_socket(socket,ic);
              raise (Interrupt "data on client_socket");
              end)
    ret;
(*  debug "test12\n" ""; *)
    let return = recieve_new str !ic in
(*  debug "get_data::recieve_return:: " return; *)
    if (get_close return) then renew_socket(socket,ic);
(*       Unix.close socket; *)
(*       debug "RETURN:::" (return^"\n\n\n"); *)
(*       close_in ic; *)
(*    debug "get_data_ret " return; *)
      return
      end

let get_data_blocking str =
  let socket = create_socket () in
    sendstr socket str;
  let return = recieve socket in
    Unix.close socket;
    return;;


  let ping () = 
    try
      let str = "HEAD / HTTP/1.0\r\n\r\n" in 
      let _ = get_data_blocking str in
      true
    with Unix_error(_,_,_)-> false;;

  let init () =
    debug "proxy" "init\n";
    let polipo_log = Filename.concat !burfiks_dir "polipo.log" in
    let polipo_cache = Filename.concat !burfiks_dir "polipo.cache" in
    verify_file polipo_log;
    verify_dir polipo_cache;
    let proxy_args = 
    match (!parent_proxy,!proxy_auth) with
    | (Some s, None)-> ("parentProxy="^s)::[]
    | (Some s, Some a)->("parentProxy="^s)::(("parentAuthCredentials="^a)::[])
    | (None,_)->  [] in
    let def_args = [ !polipo_bin;"logFile="^polipo_log;"diskCacheRoot="^polipo_cache ] in
    let polipo_args = List.append def_args proxy_args in
    polipo_pid := Some (Unix.fork ());
    if !polipo_pid = (Some 0) then
    Unix.execv !polipo_bin (Array.of_list polipo_args);
    let i=0 in
    while i<20 &(not (ping ())) do Unix.sleep 1 done

  let clear_cache () = 
    let clear_cache2 () =
    debug "proxy" "clear_cache\n";
    let polipo_log = Filename.concat !burfiks_dir "polipo.log" in
    let polipo_cache = Filename.concat !burfiks_dir "polipo.cache" in
    verify_file polipo_log;
    verify_dir polipo_cache;
    let polipo_args = [ !polipo_bin;
			"-x";
			"diskCacheUnlinkTime=0";
			"preciseExpiry=True";
			"logFile="^polipo_log;
			"diskCacheRoot="^polipo_cache ] in
    let polipo_pid2 = (Unix.fork ()) in
    if polipo_pid2 = 0 then (
    print_endline("clear cache");
    List.iter (fun s -> print_endline(s);) polipo_args;
    Unix.execv !polipo_bin (Array.of_list polipo_args);
    );
    Unix.sleep 1;
    print_endline("clear cache2") in
    try
    Mutex.lock polipo_status_mtx;
    (match !polipo_pid with 
       | Some pid -> Unix.kill pid Sys.sigint;Unix.sleep 1;
       | None -> (););
    clear_cache2 ();
    init ();
   Mutex.unlock polipo_status_mtx;
   true
   with _ ->
   Mutex.unlock polipo_status_mtx;
   false;;



  let rec always_get_data str client_socket =
    try get_data str  client_socket
    with Unix_error(err, ctx1, ctx2) as ee->
       debug (Unix.error_message err) "error\n";
(*        raise ee;; *)
       Mutex.lock polipo_status_mtx; 
      try 
      if not (ping ()) then init ();
       Mutex.unlock polipo_status_mtx; 
      always_get_data str client_socket
      with x ->
      Mutex.unlock polipo_status_mtx; 
      raise x;;


  let is_text_content url client_socket =
    let get_headers url = 
      let str = "HEAD "^url^" HTTP/1.1\r\n\r\n" in
      always_get_data str client_socket in
    let get_content_type url =
      let headers = get_headers url in
      let reg = Str.regexp_case_fold "\\(Content-Type:[ ]*\\)\\([/a-zA-Z0-9-]*\\)" in
      Str.search_forward reg headers 0;
      Str.matched_group 2 headers in
    let content_type = get_content_type url in
    let reg = Str.regexp_case_fold "\\(text/\\)\\([a-zA-Z0-9-]*\\)" in
    try Str.search_forward reg content_type 0; true
    with Not_found -> false;;

  let get_location data=
    (Str.search_forward (Str.regexp_case_fold "\\(Location:[ ]*\\)\\(.*\\)") data 0;
    let tsr = Str.matched_group 2 data in
    tsr)

  let rec get_urlbody url (((socket,ic),cl_socket,tm) as client_socket): string option =
    try
    if (is_text_content url client_socket)
    then
    begin
      let get_url url = 
        let str = "GET "^url^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n" in
        always_get_data str client_socket in
      let data = get_url url in
      let status = get_status data in
      match status with
      | "200"->
         let reg = Str.search_forward (Str.regexp "\r\n\r\n") data 0 in
        Some (Str.string_after data (reg+4))
      |  "301" | "302" ->
		let redirect=(get_location data) in
		debug "redirecting to::" "";
 		print_endline redirect;
		get_urlbody redirect client_socket
      | _ ->  None
    end
    else None 
    with | Interrupt "data on client_socket" -> raise (Interrupt "data on client_socket");
	 | Interrupt "timeout" -> raise (Interrupt "timeout");
         | e -> renew_socket(socket,ic);debug "get_urlbody" ("exception "^(Printexc.to_string e)^"\n");None


let is_text headers =
(*       let headers = get_headers url in *)
  let content_type =
    let reg = Str.regexp_case_fold "\\(Content-Type:[ ]*\\)\\([/a-zA-Z0-9-]*\\)" in
    Str.search_forward reg headers 0;
    Str.matched_group 2 headers in
  let reg = Str.regexp_case_fold "\\(text/\\)\\([a-zA-Z0-9-]*\\)" in
  try Str.search_forward reg content_type 0; true
  with Not_found -> false;;

let is_head_request str = 
  let reg = Str.regexp_case_fold "\\(^HEAD \\)" in
  try Str.search_forward reg str 0; true
  with Not_found -> false ;;

let sel2 sl timeout=
  let ret,_,_ = Unix.select sl [] [] (timeout) in
  match ret with
  |h::t -> Some h
  |[] -> None;;

    (*if ((List.length ret)=0) then
      begin 
        renew_socket(ps1,ic1);
        renew_socket(ps2,ic2);
        debug "interrupt" "timeout\n";
	match links' with
  	| ((t1',u1',l1') as tul1')::(((t2',u2',l2') as tul2')::reqt) -> 
				let req1' = ("HEAD "^u1'^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n") in
                                sendstr !ps1 req1';
				let req2' = ("HEAD "^u2'^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n") in
                                sendstr !ps2 req2';
                        	rec_send reqt (tul1::tul2::tmlinks') ((tul1',req1'),(tul2',req2'))
  	| _ -> ()
    end
    else*)

let send_all_links cls ((ps1,ic1),(ps2,ic2)) profile links send_back rank=
  let tmlinks= ref [] in
  let rec rec_send olinks' (otulr1,otulr2) timeout=
    debug "List length1:: " ((string_of_int (List.length olinks'))^"\n");
    let (otulr1' ,otulr2',links') =
    (match (otulr1,otulr2) with
    |(Some tulr1',Some tulr2')-> (Some tulr1',Some tulr2',olinks')
    |(Some tulr1',None)->
      (match olinks' with
      | ((t,u,l) as tul)::reqt ->
	let req2' = ("HEAD "^u^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n") in
        sendstr !ps2 req2';
        (Some tulr1',Some (tul,req2'),reqt)
      | [] -> (Some tulr1',None,[]))
    |(None,Some tulr2')-> 
      (match olinks' with
      | ((t,u,l) as tul)::reqt ->
	let req1' = ("HEAD "^u^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n") in
        sendstr !ps1 req1';
        (Some (tul,req1'),Some tulr2',reqt)
      | [] ->  (None,Some tulr2',[]))
    |(None,None)-> 
      (match olinks' with
      | ((t1,u1,l1) as tul1)::((t2,u2,l2) as tul2)::reqt ->
	let req1' = ("HEAD "^u1^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n") in
        sendstr !ps1 req1';
	let req2' = ("HEAD "^u2^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n") in
        sendstr !ps2 req2';
        (Some (tul1,req1'),Some (tul2,req2'),reqt)
      | ((t1,u1,l1) as tul1)::reqt -> 
	let req1' = ("HEAD "^u1^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n") in
        sendstr !ps1 req1';
        (None,None,reqt)
      | [] -> raise (Interrupt "exit"))) in
    debug "List length2:: " ((string_of_int (List.length links'))^"\n");
    debug "send_all_links" "beforesel";
    let retsock= sel2 (cls::!ps1::!ps2::[]) timeout in
    debug "send_all_links" "aftersel";(*
    let (((t1,u1,l1) as tul1),req1) = otulr1' in
    let (((t2,u2,l2) as tul2),req2) = otulr2' in*)
    match retsock with
    Some s->
    (if (s=cls) then 
              begin
		debug "interrupt" "data on client_socket\n";
                raise (Interrupt "data on client_socket");
		end


              else if (s=(!ps1)) then (
		match otulr1' with
                |Some (((t1,u1,l1) as tul1),req1) ->
                (try
		let data= recieve_new req1 !ic1 in
		if (get_close data) then renew_socket(ps1,ic1);
		if (is_head_request req1) then 
		  (if (is_text data) then 
		    (let req1' = ("GET "^u1^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n")  in
                    sendstr !ps1 req1';
		    rec_send links'  ((Some (tul1,req1')),(otulr2')) timeout)
		  else (let result =  rank profile (u1,l1) (None) in
		  send_back result tul1;rec_send links'  (None,otulr2') timeout))
		else (
      		let status = get_status data in
      		match status with
      		|  "301" | "302" ->
		  let redirect=(get_location data) in
		  debug "redirecting to::" (redirect^"\n");
		  rec_send ((t1,redirect,l1)::links')  (None,otulr2') timeout;
		| "200"->
         	  let reg = Str.search_forward (Str.regexp "\r\n\r\n") data 0 in
        	  let page = Some (Str.string_after data (reg+4)) in
		  let result =  rank profile (u1,l1) page in
		  send_back result tul1;
		  rec_send links'  (None,otulr2') timeout
      		| _ ->
		  let result =  rank profile (u1,l1) (None) in
		  send_back result tul1;
		  rec_send links'  (None,otulr2') timeout)
                with
		| End_of_file -> renew_socket(ps1,ic1);rec_send (tul1::links')  (None,otulr2') timeout;
		| Not_found-> renew_socket(ps1,ic1);rec_send (tul1::links')  (None,otulr2') timeout;)
		|None ->  renew_socket(ps1,ic1);rec_send (links')  (None,otulr2') timeout;)


              else if (s=(!ps2)) then  (
		match otulr2' with
                |Some (((t2,u2,l2) as tul2),req2) ->
                (try
		let data= recieve_new req2 !ic2 in
		if (get_close data) then renew_socket(ps2,ic2);
		if (is_head_request req2) then 
		  (if (is_text data) then 
		    (let req2' = ("GET "^u2^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n")  in
                    sendstr !ps2 req2';
		    rec_send links'  (otulr1',(Some (tul2,req2'))) timeout)
		  else (let result =  rank profile (u2,l2) (None) in
		  send_back result tul2;rec_send links'  (otulr1',None) timeout))
		else (
      		let status = get_status data in
      		match status with
      		|  "301" | "302" ->
		  let redirect=(get_location data) in
		  debug "redirecting to::" (redirect^"\n");
		  rec_send ((t2,redirect,l2)::links')  (otulr1',None) timeout;
		| "200"->
         	  let reg = Str.search_forward (Str.regexp "\r\n\r\n") data 0 in
        	  let page = Some (Str.string_after data (reg+4)) in
		  let result =  rank profile (u2,l2) page in
		  send_back result tul2;
		  rec_send links'  (otulr1',None) timeout
      		| _ ->
		  let result =  rank profile (u2,l2) (None) in
		  send_back result tul2;
		  rec_send links'  (otulr1',None) timeout)
                with
		| End_of_file -> renew_socket(ps2,ic2);rec_send (tul2::links')  (otulr1',None) timeout;
		| Not_found-> renew_socket(ps2,ic2);rec_send (tul2::links')  (otulr1',None) timeout;)
		|None ->  renew_socket(ps2,ic2);rec_send (links')  (otulr1',None) timeout;))
   | None -> debug "interrupt:timeout " "\n";
             renew_socket(ps1,ic1);
             renew_socket(ps2,ic2);
	     let head =
             match (otulr1',otulr2') with
	     | (Some (tul1,req1),Some (tul2,req2))-> tul1::tul2::[]
	     | (Some (tul1,req1),None)->tul1::[]
	     | (None,Some (tul2,req2))->tul2::[]
	     | (None,None)->[] in
             tmlinks:= (List.append head !tmlinks);
             rec_send links'  (None,None) timeout in
    try
    rec_send links  (None,None) (2.0)
    with Interrupt "exit" -> 
			renew_socket(ps1,ic1);
             		renew_socket(ps2,ic2);
			debug "interrupt:timeout " "processing tmlinks\n";
                        let links = !tmlinks in
			tmlinks:= [];
			(rec_send links  (None,None) (-1.0))
(*    match links with
  | ((t1,u1,l1) as tul1)::(((t2,u2,l2) as tul2)::reqt) -> 
				let req1 = ("HEAD "^u1^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n") in
                                sendstr !ps1 req1;
				let req2 = ("HEAD "^u2^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n") in
                                sendstr !ps2 req2;
                        	rec_send reqt [] (Some (tul1,req1),Some (tul2,req2))
  | _ -> ();;*)

(*let send_all_links cls ((ps1,ic1),(ps2,ic2)) profile links =
  let reqs = (List.map (fun l -> ("HEAD "^url^" HTTP/1.1\r\nCache-Control: max-stale=36000\r\n\r\n")) links) in
  let init pslist' reqs'=
    match (pslist',links') with
    | ((hs,hic)::ts,hl::tl)-> sock_send hs hl;hl::(init ts tl)
    | (_,_)-> rec_get pslist;[] in
  let oreqs = init pslist reqs in
  let rec_send oreqs reqs =
    let pslist' = List.map (fun (s,ic)-> !s) pslist in
    let ret,_,_ = Unix.select (cls::pslist') [] [] (-1.0) in
    List.iter2 
    (fun s -> if (s=client_socket) then 
              begin
(*                 renew_socket(socket,ic); *)
                raise (Interrupt "data on client_socket");
              end
              else
              begin
                
              end)
    ret;*)





(*  let rec is_incache url = 
    let get_incache_headers url = 
      let str = "GET "^url^" HTTP/1.0\r\nCache-Control: only-if-cached \r\n\r\n" in
      always_get_data str in
    let data = get_incache_headers url in
    let get_cache_status url =
      get_status data in
    let status = get_cache_status url in
    match status with
    | "200" -> true
    | "504" -> false
    | "302" | "301"->
		let redirect=(get_location data) in*)
		(*debug "redirecting to::" ""; 
 		print_endline redirect;*)
(*		is_incache redirect
    | _     -> false*)

(*let sort_links_cache links =
  let rec sort_links_cache_int links incache_links not_incache_links =
    match links with
    | [] ->  List.append incache_links not_incache_links
    | head :: tail -> 
	match head with
	| _,url,_ ->
	    if (is_incache url)
	    then (sort_links_cache_int tail (head::incache_links) not_incache_links)
	    else (sort_links_cache_int tail incache_links (head::not_incache_links))
	|_-> (sort_links_cache_int tail incache_links not_incache_links) 
  in
  try sort_links_cache_int links [] []
  with _ -> links*)

  (*


let test_main () =
  let socket = create_socket () in
  let ic = in_channel_of_descr socket in
  let str = "HEAD http://www.google.fr/support/news_pub/ HTTP/1.1\r\n\r\n" in
  sendstr socket str;
  let ret = recieve_new str ic in
  print_endline(str^"$$$$$$$#####");
  print_endline(ret);
  print_endline("$$$$$$$$$$$");
  let str = "GET http://www.google.fr/support/news_pub/ HTTP/1.1\r\n\r\n" in
    sendstr socket str;
  let ret = recieve_new str ic in
  print_endline(str^"$$$$$$$#####");
  print_endline(ret);
  print_endline("$$$$$$$$$$$");
  let str = "GET http://images.google.fr HTTP/1.1\r\n\r\n" in
    sendstr socket str;
  let header = recieve_new str ic in
  print_endline(str^"$$$$$$$#####");
  print_endline(header);
  print_endline("$$$$$$$$$$$");
  let str = "GET http://www.iitk.ac.in HTTP/1.1\r\n\r\n" in
  sendstr socket str;
  let header = recieve_new str ic in
  print_endline(str^"$$$$$$$#####");
  print_endline(header);
  print_endline("$$$$$$$$$$$");
   let ret = recieve_content_length header ic in 
  let str = "HEAD http://www.google.fr HTTP/1.1\r\n\r\n" in
  sendstr socket str;
  let ret = recieve_new str ic in
  print_endline(str^"$$$$$$$#####");
  print_endline(ret);
  print_endline("$$$$$$$$$$$");
  *)