(**************************************************************************)
(*  Burfiks: bayesian filter for the web                                  *)
(*                                                                        *)
(*  (c) Copyright 2007, INRIA                                             *)
(*                                                                        *)
(*  Authors:                                                              *)
(*  --------                                                              *)
(*  Francesco Zappa Nardelli (INRIA, Rocquencourt)                        *)
(*  Patala Arun Kumar (IIT Kanpur)                                        *)
(*  Sanchit Garg (IIT Bombay)                                             *)
(*  Vidyut Ghosal (IIT Kanpur)                                            *)
(*                                                                        *)
(*  This file is distributed under the terms of the GNU Public License    *)
(*  version 2, http://www.gnu.org/licenses/gpl-2.0.txt                    *)
(*                                                                        *)
(**************************************************************************)

open Unix
open Auxil
open Database
open Mail
open Rankmsg
open Proxy
open State
open Htmlscan

let opened_db = ref []

let mk_profile_name (public,user,profile) =
  if (public) then
  (let name =
    Filename.concat !State.burfiks_dir (Filename.concat "public" (Filename.concat user (profile^".db"))) in
  name)
  else
  (let name = Filename.concat !State.burfiks_dir (Filename.concat user (profile^".db")) in
  name)

let mk_temp () =
  let name =
    Filename.concat !State.burfiks_dir ("tmp.tmp") in
  name

(*let find_db profile = 
  let db_name = mk_profile_name profile in
  try 
    let db_full = (List.find (fun (x,_,_) -> (String.compare x db_name) = 0) !opened_db) in
    match db_full with
    | _,db,_ -> Some db
  with Not_found -> None*)

let find_db_full profile = 
  let db_name = mk_profile_name profile in
  try 
    let db_full = (List.find (fun (x,_,_) -> (String.compare x db_name) = 0) !opened_db) in
        Some db_full
  with Not_found -> None

let remove_db profile =
  let db_name = mk_profile_name profile in
  opened_db := List.filter (fun (x,_,_) -> not ((String.compare x db_name) = 0)) !opened_db

let open_profile profile=
  match find_db_full profile with
  | Some (db_name,db,count) -> 
       (remove_db profile;
       opened_db := (db_name,db,(count+1))::!opened_db;
       true)
  | None ->
      let db_name = mk_profile_name profile in
      try
      opened_db := ((db_name,(Database.read_full db_name),1)::!opened_db);
      true
      with Sys_error _  -> false

let create_profile profile=
  let db_name = mk_profile_name profile in
  try
    let _ = (Database.read_full db_name) in
    false
  with Sys_error _ -> Database.write_full db_name (Database.create 997);
                      true
    
let close_profile profile =
  debug "profile" ("CLOSE PROFILE: "^""^"-"^" FIX THIS "^"\n");
  match find_db_full profile with
  | Some (db_name,db,count) -> 
      if (count=1)
      then (
      remove_db profile;
      Database.write_full (mk_profile_name profile) db;)
      else (
      remove_db profile;
       opened_db := (db_name,db,(count+1))::!opened_db;)
  | None -> ()

let close_all_profiles profile =
      debug "all profiles" ("CLOSE PROFILE: "^""^"-"^" FIX THIS "^"\n");
  match find_db_full profile with
  | Some (db_name,db,count) -> 
      (remove_db profile;
      Database.write_full (mk_profile_name profile) db;)
  | None -> ()


  

let full_to_short_db db =
  { s_num_good = db.f_num_good;
    s_num_spam = db.f_num_spam;
    s_freq = db.f_high_freq }

(* let get_url = Proxy.get_urlbody *)

let no_rank = ref false

type result ={ prob: int; word_list: string }

let rank profile (url,label) page=
  let db_full = find_db_full profile in
  match db_full with
  | Some (_,db,_) -> 
      (
       match page with 
       | Some p -> 
 	  let message = 
 	    { headers = []; 
 	      body = (extract_text p); 
 	      parts = [] } in 
 	  let rank = Rankmsg.rank_message (full_to_short_db db) message in 
 	  let value = (Pervasives.truncate ((rank.spam_prob) *. 100.0)) in 
 	  debug "rank" (url ^ " : " ^ (string_of_int value) ^ "\n");
	  { prob = value;
            word_list = rank.explanation; }
      | None -> debug "rank" "no page\n";{ prob = 101;
                  word_list = ""; })
  | _ -> (debug "rank" "no db\n";{ prob = 101;
              word_list = ""; })

(* FZ this won't work with frames *)
let evaluate_url profile url is_spam page= 
  let db_full = find_db_full profile in 
  match db_full with
  | Some (_,db,_) ->
(*      let page = Proxy.get_urlbody url (socket,-1.0) in*)
      ( match page with
      | Some p ->
(*           debug "profile" ("\n\nNEW URL:: \n"^url^"##\n"^p^"\n\n\n\n"); *)
(*           debug "profile" ("\n\nNEW URL:: \n"^url^"##\n"^(extract_text p)^"\n\n\n\n"); *)
	  Processing.add_message db false is_spam (extract_text p);
          true
      | None -> false )
  | None -> false

let dump_profile profile=
 let db_full = find_db_full profile in 
  match db_full with
  | Some (_,db,_) -> 
                     let oc = open_out (mk_temp ()) in
                     Database.dump db oc;
                     close_out oc; 
                     let ic = open_in (mk_temp ()) in
                     let rec send_dump () = 
                       try
                       let str = input_line ic in
                       ("DUMP "^str^"\n")::(send_dump ())
                       with End_of_file -> close_in ic;[]
                     in
                     let head = ("DUMP_START\n"^("DUMP "^(input_line ic)^"\n")) in
                     let lstr = send_dump () in
                     let sorted_lstr = List.sort String.compare lstr in
                     let rec get_str lstr' =
                        match lstr' with
                        | [] -> "DUMP_END\n"
                        | h::t -> h^(get_str t) in
                     print_endline("head"^head);
                     head^(get_str sorted_lstr)
  | None -> ("DUMP_FAILED\n")
